
XML_NAMESPACE_FLOW_CONTROL = 'https://doculabs.io/2020/xtmpl#control'
XML_NAMESPACE_DATA_BINDING = 'https://doculabs.io/2020/xtmpl#data-binding'
XML_NAMESPACE_NOOP = 'https://doculabs.io/2020/xtmpl#noop'
XML_NAMESPACE_POJO = 'https://doculabs.io/2020/xtmpl#javascript'
XML_TEXT_BINDING_TAG = f'{{{XML_NAMESPACE_DATA_BINDING}}}text'
INDENT = '    '



from django.core.paginator import Page as BasePage, EmptyPage, Paginator as BasePaginator


class Page(BasePage):
    def next_page_number(self):
        try:
            return self.paginator.validate_number(self.number + 1)
        except EmptyPage:
            return ''

    def previous_page_number(self):
        try:
            return self.paginator.validate_number(self.number - 1)
        except EmptyPage:
            return ''


class Paginator(BasePaginator):
    def _get_page(self, *args, **kwargs):
        return Page(*args, **kwargs)



from django_kirui.context_processors import DjangoSamonBinding
from django_static_bundler.templatetags.bundled_static import bundled_static


class KiruiBinding(DjangoSamonBinding):
    def bundled_static(self, path):
        return bundled_static(path)



from .webcomponent import webcomponent
from collections import namedtuple

Attribute = namedtuple('Attibute', ['name', 'value'])


class Element:
    def __init__(self, name):
        self.tagName = name.upper()
        self._attributes = {}
        self.children = []
        self.listeners = {}

    @property
    def childNodes(self):
        return self.children

    @property
    def nodeType(self):
        return 1

    @property
    def nodeName(self):
        return self.tagName

    @property
    def attrs(self):
        return self._attributes

    @property
    def attributes(self):
        return [Attribute(name=k, value=v) for k, v in self._attributes.items()]

    def setAttribute(self, key, value):
        self._attributes[key] = str(value)

    def hasAttribute(self, key):
        return key in self._attributes.keys()

    def addEventListener(self, event, func):
        self.listeners[event] = func

    def appendChild(self, child):
        self.children.append(child)

    def __repr__(self):
        attrs = ' '.join(f'{k}="{v}"' for k, v in self._attributes.items())
        return f'<{self.tagName} {attrs}>'

    def __str__(self):
        return repr(self)


class AnonymusElement:
    def __init__(self, content):
        self.content = content

    @property
    def nodeType(self):
        return 3

    @property
    def textContent(self):
        return self.content


class Document:
    def createElement(self, node_name):
        if klass := webcomponent.get(node_name):
            return klass(node_name)
        return Element(node_name)

    def createTextNode(self, content):
        return AnonymusElement(content)



from browser import window, websocket


async_url = 'ws://' + window.DJANGO_LIVESYNC.HOST + ':' + str(window.DJANGO_LIVESYNC.PORT)


def on_message(evt):
    print(evt.data)
    print(dir(evt))


ws = websocket.WebSocket(async_url)
ws.bind('message', on_message)



import os
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = 'c(0&0l4*hsdh+#9xgtb^&a)60wdg4+umxvujrp4c+s3a@n+h(d'
DEBUG = True

ALLOWED_HOSTS = ['*']

INSTALLED_APPS = [
    # 'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',

    'livesync',
    'django_brython',
    'django.contrib.staticfiles',

    'kirui_devserver.backend',
    'django_kirui',
    'kirui',
    'django_static_bundler',
    'django_user_agents',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'livesync.core.middleware.DjangoLiveSyncMiddleware',
    'django_user_agents.middleware.UserAgentMiddleware',
]

ROOT_URLCONF = 'kirui_devserver.server.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates/html')],
        'APP_DIRS': False,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
    {
        'BACKEND': 'django_kirui.templates.samon.DjangoSamonTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates/samon')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django_kirui.context_processors.djsamon',
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    }
]

WSGI_APPLICATION = 'kirui_devserver.server.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sq3',
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

STATIC_URL = '/static/'
STATIC_ROOT = f'/{BASE_DIR}/static/'
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static")
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    #'django_brython.staticfiles.BrythonStaticGenerator',
]

BRYTHON_BUNDLER_MAIN_MODULE = 'kirui'
BRYTHON_BUNDLER_EXCLUDE = [
    'kirui.tests',
    'kirui.apps'
]

DJANGO_LIVESYNC = {
    'EVENT_HANDLER': 'django_brython.livesync.handler.DjangoBrythonEventHandler'
}

#DJANGO_LIVESYNC = {
#    'EVENT_HANDLER': 'livesync.core.handler.LiveReloadRequestHandler'
#}

DJANGO_SAMON_BINDING_CLASS = 'kirui_devserver.context_processors.KiruiBinding'
DATA_UPLOAD_MAX_NUMBER_FIELDS = None



from contextlib import contextmanager
from io import StringIO
from typing import List
from xml.sax.xmlreader import AttributesNSImpl

from . import constants
from .expressions import Bind, Condition, ForLoop, Expression
from .render import RenderedElement


class BaseElement:
    def __init__(self, xml_tag: str, xml_attrs: AttributesNSImpl):
        self.xml_tag = xml_tag

        self.xml_attrs = self._parse_xml_attrs(xml_attrs)
        self.parent = None
        self.children = []  # type: List[BaseElement]

    def _parse_xml_attrs(self, xml_attrs: AttributesNSImpl):
        attrs = {}
        for (namespace, attr_name), attr_value in xml_attrs.items():
            if namespace is None:
                attrs[attr_name] = attr_value
            else:
                key = ''.join(f'{{{n}}}' for n in namespace) + attr_name
                if constants.XML_NAMESPACE_DATA_BINDING in namespace:
                    value = Bind(expr=attr_value)
                elif namespace == (constants.XML_NAMESPACE_POJO,):
                    value = Bind(expr=attr_value)
                elif namespace == (constants.XML_NAMESPACE_FLOW_CONTROL,):
                    if attr_name == 'if':
                        value = Condition(expr=attr_value)
                    elif attr_name == 'for':
                        value = ForLoop(expr=attr_value)
                    else:
                        raise ValueError  # TODO: raise custom error
                else:
                    value = attr_value

                attrs[key] = value

        return attrs

    def add_child(self, element: 'BaseElement'):
        element.parent = self
        self.children.append(element)

    def build_rendering_tree(self, indent=1):
        yield self, indent
        for child in self.children:
            child.build_rendering_tree(indent=indent + 1)

    @property
    def attrs_as_xml(self) -> str:
        retval = ''
        for k, v in self.xml_attrs.items():
            if isinstance(v, Expression):
                v = v.expr

            k = k.replace(f'{{{constants.XML_NAMESPACE_DATA_BINDING}}}', 'b:')
            k = k.replace(f'{{{constants.XML_NAMESPACE_FLOW_CONTROL}}}', 'c:')
            retval += f' {k}="{v}"'

        return retval

    def render(self, context):
        klass = getattr(self, 'RENDERED_ELEMENT_CLASS', RenderedElement)
        return klass(self, context)


class AnonymusElement:
    def __init__(self, text):
        self.text = text
        self.xml_tag = None
        self.children = []



# Generated by Django 3.1.5 on 2021-08-14 13:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('backend', '0002_activity'),
    ]

    operations = [
        migrations.CreateModel(
            name='Reason',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
            ],
        ),
        migrations.AddField(
            model_name='activity',
            name='reason',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='backend.reason'),
        ),
    ]
